@@ -1,6 +1,6 @@
 /*****************************************************************************
  * Free42 -- an HP-42S calculator simulator
- * Copyright (C) 2004-2023  Thomas Okken
+ * Copyright (C) 2004-2024  Thomas Okken
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License, version 2,
@@ -30,7 +30,7 @@
 #define MAX_MANT_DIGITS 34
 #define ALWAYS_INT_FROM (pow(10, MAX_MANT_DIGITS))
 #else
-#define MAX_MANT_DIGITS 16
+#define MAX_MANT_DIGITS 17
 #define ALWAYS_INT_FROM ((double) (1LL << 53))
 #endif
 
@@ -82,6 +82,7 @@ class Phloat {
         Phloat operator=(uint8 i);
         Phloat operator=(double d);
         Phloat operator=(Phloat p);
+        void assign17digits(double d);
         bool operator==(Phloat p) const;
         bool operator!=(Phloat p) const;
         bool operator<(Phloat p) const;
@@ -149,6 +150,8 @@ Phloat fabs(Phloat p);
 Phloat pow(Phloat x, Phloat y);
 Phloat floor(Phloat x);
 Phloat fma(Phloat x, Phloat y, Phloat z);
+int ilogb(Phloat x);
+Phloat scalbn(Phloat x, int y);
 
 Phloat operator*(int x, Phloat y);
 Phloat operator/(int x, Phloat y);
@@ -159,8 +162,6 @@ bool operator==(int4 x, Phloat y);
 
 extern Phloat PI;
 
-void update_decimal(BID_UINT128 *val);
-
 
 #endif // BCD_MATH
 
